import {Component, EventEmitter, OnInit, ViewChild} from "@angular/core";
import {MatFormField, MatInput} from "@angular/material/input";
import {FormControl, ReactiveFormsModule} from "@angular/forms";
import {MatAutocompleteModule} from "@angular/material/autocomplete";
import {AsyncPipe} from "@angular/common";
import {catchError, map, Observable, startWith} from "rxjs";
import {LoadingIndicatorComponent} from "@component/common/loading-indicator/loading-indicator.component";
import {ProfessorResponse} from "@api/v1/professorResponse";
import {ProfessorService} from "@api/v1/professor.service";
import {IScheduleTab} from "@component/schedule/tabs/ischedule-tab";
import {TabSelect, TabStorageService} from "@service/tab-storage.service";

@Component({
  selector: 'app-professor',
  standalone: true,
  imports: [
    MatAutocompleteModule,
    MatFormField,
    AsyncPipe,
    ReactiveFormsModule,
    MatInput,
    LoadingIndicatorComponent
  ],
  templateUrl: './professor.component.html',
  styleUrl: './professor.component.css',
  providers: [ProfessorService]
})
export class ProfessorComponent implements OnInit, IScheduleTab {
  protected professorControl = new FormControl();
  protected filteredProfessors!: Observable<ProfessorResponse[]>;

  protected professors: ProfessorResponse[] | null = null;

  @ViewChild('professorIndicator') professorIndicator!: LoadingIndicatorComponent;

  public eventResult = new EventEmitter<number>();
  public selectChangeEvent = new EventEmitter<TabSelect[]>();

  constructor(private api: ProfessorService) {
  }

  getEnclosureList(): string[] {
    return ['professor'];
  }

  protected loadProfessors() {
    if (this.professors === null || this.professors.length === 0) {
      this.api.getProfessors()
        .pipe(catchError(error => {
          this.professorIndicator.loading = false;
          throw error;
        }))
        .subscribe(data => {
          this.professors = data;

          let selected = TabStorageService.selected?.selected['professor'];
          if (selected) {
            let selectedProfessor = data.find(x => x.id === selected.index);

            if (selectedProfessor === undefined || selectedProfessor.name !== selected.name)
              selectedProfessor = data.find(x => x.name === selected.name);

            if (selectedProfessor !== undefined)
              this.onOptionSelected(selectedProfessor.id);
          }
        });
    }
  }

  ngOnInit(): void {
    this.filteredProfessors = this.professorControl.valueChanges.pipe(
      startWith(''),
      map(value => this._filterProfessors(value))
    );
  }

  private _filterProfessors(value: string | number): ProfessorResponse[] {
    if (typeof value === 'string') {
      if (value === '')
        return [];

      const filterValue = value.toLowerCase().replace('ё', 'е');
      return this.professors?.filter(teacher => teacher.name.toLowerCase().replace('ё', 'е').includes(filterValue)) ?? [];
    } else {
      const selectedTeacher = this.professors?.find(teacher => teacher.id === value);
      return selectedTeacher ? [selectedTeacher] : [];
    }
  }

  protected onOptionSelected(index: number) {
    if (index === undefined)
      return;

    const selectedOption = this.professors?.find(teacher => teacher.id === index);
    if (selectedOption) {
      this.professorControl.setValue(selectedOption.name);
      this.eventResult.emit(selectedOption.id);

      this.selectChangeEvent.emit([new TabSelect(selectedOption.id, selectedOption.name)]);
    }
  }

  public load() {
    if (this.professors === null)
      this.loadProfessors();
  }
}
